from whiskerrag_types.model.knowledge import Knowledge, Task


def handle(task: Task, knowledge: Knowledge):
    # TODO: implement your code here
    print("handle task")
    print(task)
    print(knowledge)
    pass
